#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>

typedef struct _FontTable FontTable;
struct _FontTable { 
	FontTable *next; 
	FontTable *prev;
	GC gc;
	XFontSet fs;
}; 

char **missing_set, *def_string;
int missing_count;

int
fontset_height(XFontSet fs)
{
    XFontSetExtents *extent;
    extent = XExtentsOfFontSet(fs);
    return extent->max_logical_extent.height;
}

int
fontset_ascent(XFontSet fs)
{
    XFontSetExtents *extent;
    extent = XExtentsOfFontSet(fs);
    return -extent->max_logical_extent.y;
}

int
fontset_descent(XFontSet fs)
{
    XFontSetExtents *extent;
    extent = XExtentsOfFontSet(fs);
    return extent->max_logical_extent.height + extent->max_logical_extent.y;
}

static FontTable *fonttable = NULL;
XFontSet 
FontSetFromGC(GC gc)
{
	FontTable *ft = fonttable;
	while (ft) {
		if (ft->gc == gc) {
			return ft->fs;
		}
		ft = ft->next;
	}
	return NULL;
}

void
associate_gc_with_fontset(GC gc, XFontSet fs)
{
	FontTable *ft, *last;
	if (gc == NULL) return;
	ft = fonttable;
	last = NULL;
	while (ft) {
		if (ft->gc == gc) {
			/* this gc had been registered */
			if (fs) {
				/* update info */
				ft->fs = fs;
				return;
			} else {
				/* remove this item because this is no longer valid */
				if (ft->prev) {
					ft->prev->next = ft->next;
					if (ft->next) {
						ft->next->prev = ft->prev;
					}
					free(ft);
				} else {
					fonttable = ft->next;
					fonttable->prev = NULL;
				}
			}
			return;
		}
		if (ft->next) {
			ft = ft->next;
		} else {
			last = ft;
			break;
		}
	}
	if (fs == NULL) return;
	/* create the new item and append it to list */
	ft = (FontTable *)malloc(sizeof(FontTable));
	ft->prev = last;
	ft->next = NULL;
	ft->gc = gc;
	ft->fs = fs;
	if (last) {
		last->next = ft;
	} else {
		fonttable = ft;
	}
}

#if 0
#define ESC ''
void
ct2str(char *ct, char **str)
{
    int hangul = 0;
    char buf[1024];
    char *s = buf;
    while (*ct) {
	if(*ct != ESC) {
	    if(hangul) {
		*s++ = *ct++ | 0x80;
	    } else {
		*s++ = *ct++;
	    }
	} else {
	    if(strncmp(ct+1, "$(C", 3) == 0) {
		ct += 4;
		hangul = 1;
	    } else if(strncmp(ct + 1, "(B", 2) == 0) {
		ct += 3;
		hangul = 0;
	    } else if(strncmp(ct + 1, "(C", 2) == 0) {
		ct += 3;
		hangul = 1;
	    } else {
		ct++;
		// error
	    }
	}
    }
    *s = 0;
    *str = strdup(buf);
}
#endif


Status MyXGetWMName(dpy, win, winname)
Display *dpy;
Window win;
char **winname;
{
    XTextProperty text;
    char **list;
    int items;
    Status status_ret;

    if (XGetWMName(dpy, win, &text) != 0) {
	status_ret = XmbTextPropertyToTextList(dpy,&text,&list,&items);
	if (status_ret == Success && *list && items > 0) {
	    *winname = *list;
	    return 1;
	} else if (status_ret > Success) {
	    *winname = (char *)text.value;
	    return 1;
	} else
	  *winname = NULL;
    } else
      *winname = NULL;
    return 0;  
}

/*
 *  XGetIconName Wrapper
 *
 */

Status MyXGetWMIconName(dpy, win, iconname)
Display *dpy;
Window win;
char **iconname;
{
    Atom actual_type;
    XTextProperty text;
    int actual_format;
    unsigned long nitems, bytesafter;
    char **list;
    int num;
    XTextProperty text_prop;
    Status status_ret;

    if (XGetWMIconName(dpy, win, &text) != 0) {
	status_ret = XmbTextPropertyToTextList(dpy, &text_prop, &list, &num);
	if (status_ret == Success && num > 0 && *list) {
	    *iconname = *list;
	    return 1;
	} else if (status_ret > Success) {
	    *iconname = (char *)text_prop.value;
	    return 1;
	} else 
	  *iconname = NULL;
    } else
      *iconname = NULL;
    return 0;
}
